(function(pageEditorDeferRefresh, $, _) {
    var originalLoadRenderingFromUrl = null;

    pageEditorDeferRefresh.enableLogging = false;

    pageEditorDeferRefresh.triggerUpdate = function() {
        log('Triggering Page Editor update');

        var pattern = /coveoPageEditorCallback=(coveoCallback[0-9]{4})&?/;
        var result = pattern.exec(document.URL);
        if (result != null) {
            // The function name is in the first match group.
            var functionName = result[1];
            if (typeof window.top[functionName] === 'function') {
                // In the JS UI, the loading animation stays until all the components are rendered.
                // We need the setTimeout here to get the snapshot without the loading animation.
                window.setTimeout(function() {
                    window.top[functionName].apply();
                }, 0);
            }
        }
    };

    pageEditorDeferRefresh.deferredRefreshCallback = function() {
        // This method is called from a jQuery proxy. The current context
        // is the Sitecore chrome instance.

        if (this._frameLoaded) {
            log('Component is ready. Calling refresh in Sitecore.');

            // We might have lost the callback function, but we had a backup.
            if (!this._frameLoadedCallback && this.coveoFrameLoadedCallback) {
                // Restore the lost callback function.
                this._frameLoadedCallback = this.coveoFrameLoadedCallback;
            }

            this._frameLoaded();
        }
    };

    pageEditorDeferRefresh.overridePlaceholderLoadRenderingFromUrl = function() {
        // Make sure that the function have not been patched already. Patching the
        // function more than once could lead to infinite loops.
        if (typeof Sitecore !== 'undefined' && !originalLoadRenderingFromUrl) {
            log('Overriding \"_loadRenderingFromUrl\" function');

            originalLoadRenderingFromUrl = Sitecore.PageModes.ChromeTypes.Placeholder.prototype._loadRenderingFromUrl;
            Sitecore.PageModes.ChromeTypes.Placeholder.prototype._loadRenderingFromUrl = loadRenderingFromUrl;
        }
    };

    function shouldDeferChromeRefresh(scChrome) {
        // Even when editing properties on a rendering, Sitecore does not
        // pass the rendering chrome. It passes the placeholder chrome instead.
        // The trick here is to find out if a rendering with the right display name
        // has the coveoDeferRefresh flag on it. If yes, we are confident that
        // callback will be called.

        var renderings = Sitecore.PageModes.ChromeManager.getChromesByType(Sitecore.PageModes.ChromeTypes.Rendering);
        var deferRefresh = false;

        _.forEach(scChrome.element, function(element) {
            _.forEach(renderings, function(rendering) {
                if (element.innerHTML === rendering.element[0].innerHTML && rendering.data.custom.coveoDeferRefresh) {
                    deferRefresh = true;
                }
                return !deferRefresh;
            });
            return !deferRefresh;
        });

        return deferRefresh;
    }

    function loadRenderingFromUrl(url, callback) {
        log('Entering overriden _loadRenderingFromUrl function');

        var scRandom = Math.random();
        var coveoRandom = Math.floor(scRandom * 10000);

        if (shouldDeferChromeRefresh(this.chrome)) {
            if (!this._loadingFrame) {
                this._loadingFrame = $('<iframe id="scLoadingFrame"></iframe>').attr({ height: '0px', width: $(window).width() })
                                                                               .appendTo(document.body);

                // We cannot rely on the load event here because the component is initialized asynchronously.
                // It is the component's responsibility to call the callback function when it is ready.
            }

            // Sometimes, the _frameLoadedCallback attribute is set to null before our callback
            // function is called. This is quite annoying because our component does not refresh
            // properly. The workaround is to keep a backup reference on the Sitecore callback function.
            this._frameLoadedCallback = callback;
            this.coveoFrameLoadedCallback = callback;

            var coveoCallbackFunctionName = 'coveoCallback' + coveoRandom;

            // Before creating the proxy method, make sure that all properties are set
            // properly. The proxy method seems to copy the instance.
            window[coveoCallbackFunctionName] = $.proxy(pageEditorDeferRefresh.deferredRefreshCallback, this);

            this._loadingFrame[0].src = appendCallbackUrlArguments(url, scRandom, coveoCallbackFunctionName);
        } else {
            originalLoadRenderingFromUrl.call(this, url, callback);
        }
    }

    function appendCallbackUrlArguments(url, sitecoreRandomValue, coveoCallbackFunctionName) {
        log('Entering appendCallbackUrlArguments function');

        var urlBegin, urlEnd;

        var hashPosition = url.indexOf('#');
        if (hashPosition < 0) {
            // The url has no hash symbol
            urlBegin = url;
            urlEnd = '';
        } else {
            // The arguments must be inserted before the '#' symbol.
            urlBegin = url.substr(0, hashPosition);
            urlEnd = url.substr(hashPosition, url.length - hashPosition);
        }

        return urlBegin + '&rnd=' + sitecoreRandomValue + '&coveoPageEditorCallback=' + coveoCallbackFunctionName + urlEnd;
    }

    function log(message) {
        if (pageEditorDeferRefresh.enableLogging) {
            console.log(['TRACE', 'PageEditorDeferRefresh'].concat(message));
        }
    }

    // Upon page load, override the _loadRenderingFromUrl function.
    document.addEventListener('DOMContentLoaded', function() {
        pageEditorDeferRefresh.overridePlaceholderLoadRenderingFromUrl();
    });
}(Coveo.PageEditorDeferRefresh = Coveo.PageEditorDeferRefresh || {}, Coveo.$, _));
